/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.ReportParameterManager");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Displays the Export Report dialog.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.ReportParameterManager",
	null, {
	
	constructor: function(report, design, parameterDefs, paramNames, paramValues) {
		this.report = report;
		this._design = design;
		this._hasVisibleParams = false;
		if(typeof parameterDefs == "undefined" || parameterDefs == null || parameterDefs.length == 0){
			return;
		}
		this._paramsMap = new Object();
		this._paramNames = [];
		for(var i in parameterDefs){
			var def = parameterDefs[i];
			var name = def.name;
			this._paramNames.push(name);
			var param = def;
			this._paramsMap[name] = param;
			var value = this._getInitialValue(def, paramNames, paramValues);
			param.currentValue = value;
			if(value instanceof Array) param.initialValue = this._copyArray(value);
			else param.initialValue = value;
			if(this.isHidden(name)) continue;
			this._hasVisibleParams = true;
			if(def.selectionListType != "NONE"){
				if(param.groupName == null || param.groupName == "")
					this._fetchSelectionList(name);
				else
					this._fetchCascadingParameterSelectionList(i);
			}
		}
	},
	
	hasVisibleParams: function() {
		return this._hasVisibleParams;
	},
	
	hasAllRequiredValues: function() {
		for (var i in this._paramNames){
			var name = this._paramNames[i];
			var param = this._paramsMap[name];
			var value = param.currentValue;
			if(param.required && (typeof value == "undefined" || value == null || value == "")) return false;
		}
		return true;
	},
	
	getParamNames: function() {
		return this._paramNames;
	},
	
	isHidden: function(name) {
		//TODO check special parameter
		var param = this.getParamByName(name);
		if(param == null) return true;
		return param.isHidden;
	},
	
	getDisplayName: function(name) {
		var param = this.getParamByName(name);
		if(param == null) return "";
		if(param.displayName != null) return param.displayName;
		return param.name;
	},
	
	getCurrentValueLabel: function(name) {
		var param = this.getParamByName(name);
		if(param == null || param.currentValue == null || param.currentValue == "") return ReportMessages.REPORT_PARAM_NO_SELECTION_TEXT;
		if(param.selectionListType == "NONE"){
			if(param.dataType == "DATE" || param.dataType == "TIME")
				return this._formatDateTime(param.dataType, param.currentValue);
			return param.currentValue;
		} else {
			if(typeof param.choices == "undefined" || param.choices == null)
				return param.currentValue;
			var value = param.currentValue;
			if(value instanceof Array){
				if(value.length > 1) return value.length + ReportMessages.REPORT_PARAM_SELECTIONS_TEXT;
				else value = value[0];
			}
			var index = dojo.indexOf(param.choices.values, value);
			if(index != -1) return param.choices.labels[index];
			else return value;
		}
	},
	
	updateCurrentValue: function(name, value) {
		var param = this.getParamByName(name);
		param.currentValue = value;
		//for cascading parameters
		if(param.groupName != null && param.groupName != ""){
			this._resetCascadingParameters(name);
		}
	},
	
	resetCurrentValues: function() {
		var groupName = null;
		var groupChanged = false;
		for (var i in this._paramNames){
			var name = this._paramNames[i];
			var param = this._paramsMap[name];
			if (typeof param.groupName == "undefined" || param.groupName == null || param.groupName == "") 
				groupName = null;
			else 
				if (param.groupIndex == 0) {
					groupName = param.groupName;
					groupChanged = false;
				}
			var currentValue = param.currentValue;
			param.currentValue = param.initialValue;
			if(groupName != null && groupChanged)
				this._fetchCascadingParameterSelectionList(i);
			if(groupName != null && currentValue != param.initialValue)
				groupChanged = true;
		}
	},
	
	getParamByName: function(name) {
		var param = this._paramsMap[name];
		if(typeof param == "undefined" || param == null) return null;
		return param;
	},
	
	getParamNameAndValues: function() {
		var parameters = {
			names: [],
			values: []
		};
		for (var i in this._paramNames){
			var name = this._paramNames[i];
			var param = this._paramsMap[name];
			var value = param.currentValue;
			if(typeof value == "undefined" || value == null || value == "") continue;
			if(value instanceof Date){
				value = value.getTime();
			}
			parameters.names.push(name);
			parameters.values.push(this._wrapValue(value));
		}
		return parameters;
	},
	
	_fetchSelectionList: function(name){
		var args = {
			id: this.report.itemId,
			parameterName: name
		};
		if(typeof this.report.reportDesign != "undefined" && this.report.reportDesign != null){
			args.id = this.report.reportDesign.itemId;
		}
		if (this._design != null) {
			args.design = this._design;
		}
		var srh = new ServiceResponseHandler(this, "_saveSelectionList", "_remoteError");
		ReportClient.getReportParameterValues(srh, args);
	},
	
	_saveSelectionList: function(values) {
		this._paramsMap[values.name].choices = {
			labels: [],
			values: []
		};
		if(typeof values.labels != "undefined" && values.labels != null)
			this._paramsMap[values.name].choices.labels = values.labels;
		if(typeof values.values != "undefined" && values.values != null)
			this._paramsMap[values.name].choices.values = values.values;
		dojo.publish("com.ibm.rational.report.ReportParameterSelectionList."+values.name);
	},
	
	_getInitialValue: function(paramDef, paramNames, paramValues) {
		var name = paramDef.name;
		//use the value specified in the request if its specified
		if(typeof paramNames != "undefined" && paramNames != null && 
			typeof paramValues != "undefined" && paramValues != null){
			if(paramNames instanceof Array){
				var index = dojo.indexOf(paramNames, name);
				if(index != -1)	{
					var value = paramValues[index];
					if(typeof value != "undefined" && value != null)
						return this._parseValue(paramDef.dataType, value);
				}
			}
			else if (name == paramNames && typeof paramValues != "undefined" && paramValues != null)
				return paramValues;
		}
		//or else, use the value stored in the report descriptor if this is a report query
		var reportParameters = this.report.reportParameters;
		if(typeof reportParameters != "undefined" && reportParameters != null){
			for(var i in reportParameters){
				var param = reportParameters[i];
				if(param.key != name) continue;
				if(typeof param.value != "undefined" && param.value != null)
					return this._parseValue(paramDef.dataType, param.value);
			}
		}
		//or else, use the default value specified in the report parameter definition.
		return this._parseValue(paramDef.dataType, paramDef.defaultValue);
	},
	
	_fetchCascadingParameterSelectionList: function(index){
		var currentParamName = this._paramNames[index];
		var groupName = this._paramsMap[currentParamName].groupName;
		var groupIndex = this._paramsMap[currentParamName].groupIndex;
		var currentValues = [];
		var startIndex = index - groupIndex;
		for (var i=startIndex; i<index; i++) {
			var paramName = this._paramNames[i];
			var param = this._paramsMap[paramName];
			if(typeof param.currentValue == "undefined" || param.currentValue == null || param.currentValue == "" || param.currentValue == [])
				return;
			currentValues.push(param.currentValue);
		};
		var args = {
			id: this.report.itemId,
			groupName: groupName,
			values: currentValues
		};
		if(typeof this.report.reportDesign != "undefined" && this.report.reportDesign != null){
			args.id = this.report.reportDesign.itemId;
		}
		if (this._design != null) {
			args.design = this._design;
		}
		var srh = new ServiceResponseHandler(this, "_saveSelectionList", "_remoteError");
		ReportClient.getCascadingParameterValues(srh, args);
	},
	
	_resetCascadingParameters: function(currentParamName) {
		var index = dojo.indexOf(this._paramNames, currentParamName);
		var groupName = this._paramsMap[currentParamName].groupName;
		var groupIndex = this._paramsMap[currentParamName].groupIndex;
		//clear value and labels of all following parameters in the same cascading group
		var hasNext = false;
		for (var i=index+1; i<this._paramNames.length; i++) {
			var paramName = this._paramNames[i];
			var param = this._paramsMap[paramName];
			if(param.groupName == groupName){
				hasNext = true;
				param.currentValue = param.initialValue;
				param.choices = null;
				dojo.publish("com.ibm.rational.report.ReportParameterSelectionList."+paramName);
			} else break;
		}
		if(!hasNext) return;
		this._fetchCascadingParameterSelectionList(index+1);
	},
	
	_parseValue: function(dataType, value) {
		if(typeof value == "undefined" || value == null) return null;
		var values = value.split(",");
		if(values.length == 1) {
			if(dataType == "DATE" || dataType == "TIME"){
				var date = new Date();
				var time = this._stripQuote(values[0]);
				date.setTime(time);
				return date;
			}
			else return this._stripQuote(values[0]);
		}
		for(var i in values){
			values[i] = this._stripQuote(values[i]);
		}
		return values;
	}, 
	
	_formatDateTime: function(type, value){
		var newvalue = value;
		if(type == "DATE")
			newvalue = dojo.date.locale.format(value, {selector: "date"});
		else if(type == "TIME")
			newvalue = dojo.date.locale.format(value, {selector: "time"});
//		else value = dojo.date.locale.format(value);
		return newvalue;
	},
	
	_stripQuote: function(value) {
		if(value.charAt(0)=="'") value = value.substring(1,value.length);
		if(value.charAt(value.length-1)=="'") value = value.substring(0, value.length-1);
		return value;
	},
	
	_wrapValue: function(value) {
		if(value instanceof Array){
			var newValue = [];
			for(var i in value){
				newValue[i] = "'"+value[i]+"'";
			}
			return newValue.toString();
		}
		return "'"+value+"'";
	},
	
	_copyArray: function(value) {
		var newvalue = [];
		for(var i in value) newvalue[i] = value[i];
		return newvalue;
	},
	
	_remoteError: function(error) {
		this._displayErrorMessage(error);
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	}

});

})();