/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.ReportParameterWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TimeTextBox");
dojo.require("dojo.date.locale");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var InternalMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var Util					= com.ibm.rational.report.web.core.app.internal.Util;

/**
 * Displays the Export Report dialog.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.ReportParameterWidget",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/ReportParameterWidget.html"),
	
	constructor: function(args) {
		this._paramManager = args.paramManager;
		this._paramName = args.paramName;
		this._inEditMode = false;
	},
	
	postCreate: function() {
		this.displayInReadMode();
		var self = this;
		dojo.subscribe("com.ibm.rational.report.ReportParameterSelectionList."+this._paramName, 
			function() {
				self._updateLabel();
			}
		);
	},
	
	displayInReadMode: function() {
		this._inEditMode = false;
		this._removeContents();
		this.labelDiv = document.createElement("div");
		this.labelDiv.appendChild(document.createTextNode(this._paramManager.getDisplayName(this._paramName)+":"));
		this.labelDiv.className = "labelLightText";
		this.readModeContainer.appendChild(this.labelDiv);
		this.valueDiv = document.createElement("div");
		var value = this._paramManager.getCurrentValueLabel(this._paramName);
//		if(value instanceof Date)
//			value = dojo.date.locale.format(value, {selector: "date"});
		this.valueDiv.appendChild(document.createTextNode(value));
		this.valueDiv.className = "paramValueLightText";
		this.readModeContainer.appendChild(this.valueDiv);
	},
	
	displayInEditMode: function() {
		var self = this;
		this._inEditMode = true;
		this._removeContents();
		var param = this._paramManager.getParamByName(this._paramName);
		this.labelDiv = document.createElement("div");
		this.labelDiv.appendChild(document.createTextNode(this._paramManager.getDisplayName(this._paramName)+":"));
		this.labelDiv.className = "labelDarkText";
		if(param.required){
			var requiredMark = document.createElement("span");
			requiredMark.appendChild(document.createTextNode("*"));
			requiredMark.className = "requiredMarker";
			this.labelDiv.appendChild(requiredMark);
		}
		if(param.selectionListType != "NONE" && param.allowMultipleValues){
			var selectAll = document.createElement("a");
			selectAll.className = "selectDeselectLink";
			selectAll.appendChild(document.createTextNode(ReportMessages.REPORT_PARAM_SELECT_ALL_TEXT));
			selectAll.href = "javascript: {}";		
			dojo.connect(selectAll, "onclick",
				function(e) {
					e.preventDefault();
					self._selectionAll(true);
				}
			);
			var sep = document.createElement("a");
			sep.className = "separator";
			var deselectAll = document.createElement("a");
			deselectAll.className = "selectDeselectLink";
			deselectAll.appendChild(document.createTextNode(ReportMessages.REPORT_PARAM_SELECT_NONE_TEXT));
			deselectAll.href = "javascript: {}";
			dojo.connect(deselectAll, "onclick",
				function(e) {
					e.preventDefault();
					self._selectionAll(false);
				}
			);
			if(dojo.isIE){
				this.editModeContainer.appendChild(this.labelDiv);
				this.labelDiv.appendChild(sep);
				this.labelDiv.appendChild(document.createTextNode(ReportMessages.REPORT_PARAM_SELECT_TEXT));
				this.labelDiv.appendChild(selectAll);
				var sep = document.createElement("a");
				sep.className = "separator";
				this.labelDiv.appendChild(sep);
				this.labelDiv.appendChild(deselectAll);
			} else {
				var div = document.createElement("div");
				this.editModeContainer.appendChild(div);
				var table = document.createElement("table");
				table.width = "100%";
				table.cellSpacing = "0";
				table.cellPadding = "0";
				table.className = "compactTable";
				div.appendChild(table);
				var tr = document.createElement("tr");
				table.appendChild(tr);
				var td1 = document.createElement("td");
				td1.appendChild(this.labelDiv);
				tr.appendChild(td1);
				var td2 = document.createElement("td");
				tr.appendChild(td2);
				td2.align = "right";
				td2.appendChild(document.createTextNode(ReportMessages.REPORT_PARAM_SELECT_TEXT));
				td2.appendChild(selectAll);
				td2.appendChild(sep);
				td2.appendChild(deselectAll);
			}
		}
		else this.editModeContainer.appendChild(this.labelDiv);
		this.valueDiv = document.createElement("div");
		this.editModeContainer.appendChild(this.valueDiv);
		if(param.selectionListType != "NONE" && param.allowMultipleValues){
			this.list = document.createElement("div");
			if (dojo.isIE) {
				this.list.className = "multiCheckboxListIe";
			} else if (dojo.isMozilla) {
				this.list.className = "multiCheckboxList";
			}	
			this.valueDiv.appendChild(this.list);
			this._updateLabel();
			return;
		}
		if(param.selectionListType != "NONE" && !param.allowMultipleValues){
			this.list = document.createElement("div");
			this.list.className = "singleSelectCombo";			
			this.selected = document.createElement("div");
			this.selected.className = "editableSelect";
			this.list.appendChild(this.selected);
			this.select = document.createElement("select");
			this.select.size = 9;
			this.select.className = "singleSelectList";
			this.list.appendChild(this.select);
			this.valueDiv.appendChild(this.list);
			this._updateLabel();
			dojo.connect(this.select, "onchange",
				function(e) {
					e.preventDefault();
					self._selectionChanged();
				}
			);
			return;
		}
		if(param.dataType == "DATE"){
			this.text = new dijit.form.DateTextBox();
			if(typeof param.currentValue != "undefined" && param.currentValue != null)
				this.text.setValue(param.currentValue);
			this.valueDiv.appendChild(this.text.domNode);
			dojo.connect(this.text, "onChange",
				function(e) {
					self._dateChanged();
				}
			);
			return;
		}
		if(param.dataType == "TIME"){
			this.text = new dijit.form.TimeTextBox();
			if(typeof param.currentValue != "undefined" && param.currentValue != null)
				this.text.setValue(param.currentValue);
			this.valueDiv.appendChild(this.text.domNode);
			dojo.connect(this.text, "onChange",
				function(e) {
					self._dateChanged();
				}
			);
			return;
		}
		this.text = document.createElement("input");
		this.valueDiv.appendChild(this.text);
		var type = "text";
		if(param.isConcealed) type = "password";
		this.text.type = type;
		this.text.className = "text";
		if(typeof param.currentValue != "undefined" && param.currentValue != null)
			this.text.value = param.currentValue;
		dojo.connect(this.text, "onchange",
			function(e) {
				e.preventDefault();
				self._textChanged();
			}
		);
	},
	
	mouseOver: function() {
		this.labelDiv.className = "labelDarkText";
		this.valueDiv.className = "paramValueDarkText";
	},
	
	mouseOut: function() {
		this.labelDiv.className = "labelLightText";
		this.valueDiv.className = "paramValueLightText";
	},
	
	_removeContents: function() {
		Util.removeChildren(this.readModeContainer);
		Util.removeChildren(this.editModeContainer);
	},
	
	_updateLabel: function() {
		var self = this;
		if(this._inEditMode){
			var param = this._paramManager.getParamByName(this._paramName);
			if(param.selectionListType != "NONE" && param.allowMultipleValues){
				if(typeof param.choices == "undefined" || param.choices == null){
					this.list.appendChild(document.createTextNode(InternalMessages.LOADING_PROGRESS_MSG));
					return;
				}
				Util.removeChildren(this.list);
				this.checkboxs = [];
				for(var i in param.choices.labels) {
					var opt = document.createElement("div");
					opt.className = "multiCheckboxListInput";
					this.checkboxs[i] = new dijit.form.CheckBox({name: i, type: "checkbox"});
					opt.appendChild(this.checkboxs[i].domNode);
					opt.appendChild(document.createTextNode(param.choices.labels[i]));
					this.list.appendChild(opt);
					var value = param.choices.values[i];
					if(typeof param.currentValue != "undefined" && param.currentValue != null && 
						dojo.indexOf(param.currentValue, value) != -1){
						this.checkboxs[i].setChecked(true);
					}
					dojo.connect(this.checkboxs[i], "onClick",
						function(e) {
							e.preventDefault();
							self._checkboxClicked(e.target.name);
						}
					);
				}
				return;
			}
			if(param.selectionListType != "NONE" && !param.allowMultipleValues){	
				if(typeof param.choices == "undefined" || param.choices == null){
					Util.removeChildren(this.selected);
					this.selected.appendChild(document.createTextNode(InternalMessages.LOADING_PROGRESS_MSG));
					return;
				}
				var currentLabel = null;
				Util.removeChildren(this.select);
				for(var i in param.choices.labels) {
					var opt = document.createElement("option");
					opt.value = param.choices.values[i];
					opt.appendChild(document.createTextNode(param.choices.labels[i]));
					this.select.appendChild(opt);
					if(param.choices.values[i] == param.currentValue){
						opt.selected = true;
						currentLabel = param.choices.labels[i];
					}
				}
				if(currentLabel == null) currentLabel = ReportMessages.REPORT_PARAM_NO_SELECTION_TEXT;
				Util.removeChildren(this.selected);
				this.selected.appendChild(document.createTextNode(currentLabel));
				return;
			}	
		} else {
			Util.removeChildren(this.valueDiv);
			var value = this._paramManager.getCurrentValueLabel(this._paramName);
			this.valueDiv.appendChild(document.createTextNode(value));
		}
	},
	
	_checkboxClicked: function(id){
		var param = this._paramManager.getParamByName(this._paramName);
		var value = param.currentValue;
		if(typeof value == "undefined" || value == null)
			value = [];
		if(this.checkboxs[id].checked) value.push(param.choices.values[id]);
		else {
			var index = value.indexOf(param.choices.values[id]);
			if (index != -1) value.splice(index, 1);
		}
		this._updateValue(value);
	}, 
	
	_selectionChanged: function() {
		var value = this.select.value;
		var label = value;
		var param = this._paramManager.getParamByName(this._paramName);
		var index = dojo.indexOf(param.choices.values,value);
		if(index != -1) label = param.choices.labels[index];
		Util.removeChildren(this.selected);
		this.selected.appendChild(document.createTextNode(label));
		this._updateValue(value);
	},
	
	_selectionAll: function(checked) {
		if(typeof this.checkboxs == "undefined" || this.checkboxs == null) return;
		var param = this._paramManager.getParamByName(this._paramName);
		var value = [];
		for (var i in this.checkboxs) {
			this.checkboxs[i].setChecked(checked);
			if(checked) value.push(param.choices.values[i]);
		}
		this._updateValue(value);
	},
	
	_textChanged: function() {
		this._updateValue(this.text.value);
	},
	
	_dateChanged: function() {
		this._updateValue(this.text.getValue());
	},
	
	_updateValue: function(value) {
		this._paramManager.updateCurrentValue(this._paramName, value);
	}

});

})();